#!/bin/bash

# Start engine-core through pipewire
echo "Starting engine-core with pw-jack..."
cd src && /usr/bin/pw-jack /usr/bin/liquidsoap ./engine.liq &

# Capture the PID of the liquidsoap process
LIQUIDSOAP_PID=$!

# wait and link ports
sleep 5
# TODO: change aura LQ ports when we rename them

AURA_ENGINE_LQ_LINE_OUT_LEFT="aura_engine_line_out_0:out_0"
AURA_ENGINE_LQ_LINE_OUT_RIGHT="aura_engine_line_out_0:out_1"
AURA_ENGINE_LQ_LINE_IN_LEFT="aura_engine_line_in_0:in_0"
AURA_ENGINE_LQ_LINE_IN_RIGHT="aura_engine_line_in_0:in_1"

# Function to link ports and print error message if it fails
link_ports() {
    local source_port=$1
    local target_port=$2
    if pw-link "$source_port" "$target_port" 2>/dev/null; then
        echo "Linked '${source_port}' - '${target_port}'"
    else
        echo "Failed to link '${source_port}' - '${target_port}': No such file or directory"
    fi
}

# Check if environment variables are set before linking ports
if [[ -n "$AURA_ENGINE_INPUT_CHANNEL_LEFT" ]]; then
    link_ports "$AURA_ENGINE_LQ_LINE_OUT_LEFT" "$AURA_ENGINE_INPUT_CHANNEL_LEFT"
else
    echo "Environment variable AURA_ENGINE_INPUT_CHANNEL_LEFT is not set. Skipping link for '${AURA_ENGINE_LQ_LINE_OUT_LEFT}'."
fi

if [[ -n "$AURA_ENGINE_INPUT_CHANNEL_RIGHT" ]]; then
    link_ports "$AURA_ENGINE_LQ_LINE_OUT_RIGHT" "$AURA_ENGINE_INPUT_CHANNEL_RIGHT"
else
    echo "Environment variable AURA_ENGINE_INPUT_CHANNEL_RIGHT is not set. Skipping link for '${AURA_ENGINE_LQ_LINE_OUT_RIGHT}'."
fi

if [[ -n "$AURA_ENGINE_OUTPUT_CHANNEL_LEFT" ]]; then
    link_ports "$AURA_ENGINE_OUTPUT_CHANNEL_LEFT" "$AURA_ENGINE_LQ_LINE_IN_LEFT"
else
    echo "Environment variable AURA_ENGINE_OUTPUT_CHANNEL_LEFT is not set. Skipping link for '${AURA_ENGINE_LQ_LINE_IN_LEFT}'."
fi

if [[ -n "$AURA_ENGINE_OUTPUT_CHANNEL_RIGHT" ]]; then
    link_ports "$AURA_ENGINE_OUTPUT_CHANNEL_RIGHT" "$AURA_ENGINE_LQ_LINE_IN_RIGHT"
else
    echo "Environment variable AURA_ENGINE_OUTPUT_CHANNEL_RIGHT is not set. Skipping link for '${AURA_ENGINE_LQ_LINE_IN_RIGHT}'."
fi

# Wait for the liquidsoap process to exit
wait $LIQUIDSOAP_PID

# Exit with status of the liquidsoap process
exit $?